//=============================================================================
// TriggeredDisease.
// When triggered, makes the instigator get sick.
//=============================================================================
class TriggeredDisease expands Triggers;

//=============================================================================
// Configurable variables
var() int	CatchHealthLimit;				// If healthier than this, won't be infected
var() int   DiseaseDamage;					// How much damage does the disease do
var() float DiseaseInterval;				// How often does the damage occur
var() int   CureHealthLimit;				// If not zero, exceeding this amount of health cures
var() name  DiseaseType;					// Just a name for the disease damage
var() localized  string  DiseaseString;		// Disease damage string
var() sound	InfectionSound;					// Played when infected
var() sound	DisinfectionSound;				// Played when cured
var() bool	bFirepowerDegration;			// If true degrades Your firepower as You get sicker
var() bool	bContagiousDisease;				// If true, the disease is contagiuos

//=============================================================================
// Local variables
var Pawn  				Victim[16];			// We assume that there may only be 16 patients
var float       		VictimTime[16];		// Used by the interval system

//=============================================================================
// Functions
function MakeSickLooking( Pawn P ) {
	local int	hh;

	hh = 100 - P.Health;
	if	( hh <= 0 )	 hh = 1;
	if	( hh > 100 ) hh = 100;

	P.AmbientGlow = 2 * hh; 
	P.LightEffect=LE_NonIncidence;
	P.LightBrightness= 2 * hh;
	P.LightHue=80;
	P.LightRadius= hh / 5 + 2;
	P.LightSaturation=0;
	P.LightType=LT_Steady;
	if	( bFirepowerDegration ) {
		P.DamageScaling = 1.0 - hh / 100;
	}
}

function MakeNormalLooking( Pawn P ) {
	P.AmbientGlow 	= P.Default.AmbientGlow;
	P.LightType		= LT_None;
	P.DamageScaling = 1.0;
}

function	bool	AddNewPatient( Pawn P ) {
	local int		vl;
	local Inventory	ii;

	// Make sure it is a player or a bot
	if	( P == None )				return	false;
	if	( ( ! P.IsA( 'PlayerPawn' ) ) &&
		  ( ! P.IsA( 'Bots' ) ) &&
		  ( ! P.IsA( 'Bot' ) ) )	return	false;

	// Make sure we have not listed this one already
	for	( vl = 0; vl < 16; vl++ ) {
		if	( Victim[vl] == P ) 	return	false;
	}

	// Add it to our list. Use last slot in case they all are occupied.
	for	( vl = 0; vl < 15; vl++ ) {
		if	( Victim[vl] == None ) {
			break;
		}
	}
	Victim[vl] 		= P;
	VictimTime[vl]	= 0;

	if	( InfectionSound != None ) P.PlaySound( InfectionSound );

	// Remove all armors including the shield belt
	for	( ii = P.Inventory; ii != None; ii = ii.Inventory ) {
		if ( ii.bIsAnArmor ) {
			ii.Destroy();
		}
	}

	// Cause damage enough to bring health under cure limit
	if	( ( P.Health >= CureHealthLimit ) && ( CureHealthLimit > 0 ) ) {
       	Level.Game.SpecialDamageString = DiseaseString;
    	P.TakeDamage( P.Health - CureHealthLimit + 1, P, P.Location, Vect(0,0,0), DiseaseType );
	}

	return	true;
}

function	TransmitDisease( Pawn P ) {
	local	Pawn	pp;
	for ( pp = Level.PawnList; pp != None; pp = pp.NextPawn ) {
		if	( pp == P )	continue;
		if	( ( ! pp.IsA( 'PlayerPawn' ) ) &&
			  ( ! pp.IsA( 'Bots' ) ) &&
			  ( ! pp.IsA( 'Bot' ) ) )	continue;

		if	( ( abs( P.Location.Z - pp.Location.Z ) < P.CollisionHeight + pp.CollisionHeight ) &&
			  ( VSize( P.Location - pp.Location ) < P.CollisionRadius * 4.0 ) ) {
			AddNewPatient( pp );
		}
	}
}

//=============================================================================
// States
auto state  Enabled {

    function    Trigger( actor Other, pawn EventInstigator ) {
		local Pawn			pp;

		// Check out if this player is too healthy to catch this disease
		pp = Pawn( Other );
		if	( pp == None ) return;

		if	( ( pp.Health >= CatchHealthLimit ) &&
			  ( CatchHealthLimit > 0 ) ) {
			return;
		}

		// The add patient function deals with duplicates
		if	( AddNewPatient( pp ) )	Enable( 'Tick' );
    }

	function    Tick( float DeltaTime ) {
		local	Pawn	pp;
		local	Actor	aa;
	    local   int     vl;
		local	int		il;
	    local   bool    bVictimsLeft;

	    for ( vl = 0; vl < 16; vl++ ) {

	        // Empty patient slots are skipped
	        if  ( Victim[vl] == None ) {
	            continue;
	        }

			pp = Victim[vl];

	        // Did the victim cure itself ?
	        if  ( ( CureHealthLimit > 0 ) &&
	              ( pp.Health >= CureHealthLimit ) ) {
				MakeNormalLooking( pp );
	            Victim[vl] = None;
				if	( DisinfectionSound != None ) pp.PlaySound( DisinfectionSound );
	            continue;
	        }

	        // Is the victim dead already ?
	        if  ( pp.Health <= 0 ) {
				MakeNormalLooking( pp );
	            Victim[vl] = None;
				VictimTime[vl] = 0;
	            continue;
	        }

	        // Add to the patient's timer interval counter
	        bVictimsLeft = true;
	        VictimTime[vl] += DeltaTime;

	        // If the time is right for damage do some
	        if  ( VictimTime[vl] >= DiseaseInterval ) {
		        VictimTime[vl] -= DiseaseInterval;

		        // If the patient would die now, kill him
		        if  ( pp.Health <= DiseaseDamage ) {
				    Level.Game.SpecialDamageString = DiseaseString;
					pp.health = -1000;
            	    pp.Died( None, DiseaseType, pp.Location );
					MakeNormalLooking( pp );
	            	Victim[vl] = None;
		            continue;
		        }

		        // Just damage the patient
    	        Level.Game.SpecialDamageString = DiseaseString;
			    pp.TakeDamage( DiseaseDamage, None, pp.Location, Vect(0,0,0), DiseaseType );
				MakeSickLooking( pp );
	        }

			// If contagious, try transmitting it to nearby pawns
			if	( bContagiousDisease )	TransmitDisease( pp );
	    }

	    // If no more victims were found, we disable the tick
	    if  ( ! bVictimsLeft )  Disable( 'Tick' );
		else					Enable( 'Tick' );
	}
}

defaultproperties
{
     DiseaseDamage=5
     DiseaseInterval=1.000000
     CureHealthLimit=105
     DiseaseString="%o was killed by Nali Plague"
     InfectionSound=Sound'Botpack.Boss.BInjur1'
     DisinfectionSound=Sound'Botpack.Boss.BNewGib'
     bFirepowerDegration=True
}
